"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNodePackageInfo = void 0;

var _path = require("path");

var _toolLib = require("@kettil/tool-lib");

var _licenseHeuristics = require("../../configs/licenseHeuristics");

var _access = require("../cmd/access");

var _exec = require("../cmd/exec");

var _readdir = require("../cmd/readdir");

var _readFile = require("../cmd/readFile");

const regexpFilename = /^(?:copying|licen[cs]e|licen[cs]e-\w+|readme)(?:\.markdown|\.md|\.txt)?$/iu;

const getLicenseFromFiles = async ({
  path,
  data
}) => {
  const entries = await (0, _readdir.readdir)(path);
  const files = entries.filter(entry => entry.isFile() && !entry.name.startsWith('.') && regexpFilename.test(entry.name)).map(entry => entry.name).sort(); // eslint-disable-next-line no-restricted-syntax -- the loop should be canceled after the first find

  for (const file of files) {
    // eslint-disable-next-line no-await-in-loop -- the loop should be canceled after the first find
    const content = await (0, _readFile.readFile)((0, _path.join)(path, file)); // eslint-disable-next-line no-restricted-syntax -- the loop should be canceled after the first find

    for (const [aliasLicense, regexp] of (0, _toolLib.objectEntries)(_licenseHeuristics.licenseHeuristics)) {
      if (regexp.test(content)) {
        return { ...data,
          license: aliasLicense
        };
      }
    }
  }

  return undefined;
};

const getNodePackageInfo = async ({
  cwd,
  log
}, {
  path,
  ignorePackages,
  licensePackages
}) => {
  const packageJsonPath = (0, _path.join)(path, 'package.json');
  const hasPackageJson = await (0, _access.access)(packageJsonPath);

  if (!hasPackageJson) {
    // No node.js package
    return undefined;
  }

  const {
    name,
    license,
    version
  } = await (0, _readFile.readFile)(packageJsonPath, true);

  if (typeof name !== 'string') {
    throw new TypeError(`Package name is not defined (${packageJsonPath})`);
  }

  if (ignorePackages.includes(name)) {
    return undefined;
  }

  const data = {
    path: path.slice(Math.max(0, cwd.length + 1)),
    version: typeof version === 'string' ? version : 'UNKNOWN',
    license: 'UNKNOWN',
    name
  }; // Manual license assignment

  const licensePackage = licensePackages[data.name];

  if ((0, _toolLib.isObject)(licensePackage) && typeof licensePackage[data.version] === 'string') {
    return { ...data,
      license: licensePackage[data.version]
    };
  } // License from the package.json


  if (typeof license === 'string') {
    return { ...data,
      license
    };
  } // License from the package.json


  if ((0, _toolLib.isObject)(license) && typeof license.type === 'string') {
    return { ...data,
      license: license.type
    };
  } // License from copying|license|readme file


  const fileLicense = await getLicenseFromFiles({
    path,
    data
  });

  if (fileLicense) {
    return fileLicense;
  } // License over "npm view"


  const {
    stdout: npmViewLicense
  } = await (0, _exec.exec)({
    cmd: 'npm',
    args: ['view', name, 'license'],
    log,
    cwd
  });

  if (npmViewLicense.trim() !== '') {
    return { ...data,
      license: npmViewLicense.trim()
    };
  }

  return data;
};

exports.getNodePackageInfo = getNodePackageInfo;